package gz.cyberbrain.cyberhotel.model.util.dates;

import gz.cyberbrain.cyberhotel.view.swing.util.gui.StringUtils;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.GregorianCalendar;

public final class DateOperations {

	private DateOperations() {}

	/**
	 * Returns a copy of the date passed as a parameter with the
	 * <code>Calendar.MILLISECOND</code> field set to 0.
	 */
	public final static Calendar getDateWithoutMilliseconds(Calendar date) {
		/*
		 * ***WARNING***
		 *
		 * An alternative implementation of this method could be:
		 *
		 * Calendar dateWithoutMilliseconds = date.clone();
		 * dateWithoutMilliseconds.set(Calendar.MILLISECOND, 0);
		 * return dateWithoutMilliseconds;
		 *
		 * However, if the date passed as a parameter comes from a remote
		 * invocation (RMI, for example), the clone gets corrupted after the 
		 * invocation of the "set" method. Incredibly, but true !!!
		 */

		Calendar dateWithoutMilliseconds = Calendar.getInstance();

		dateWithoutMilliseconds.setTime(date.getTime());
		dateWithoutMilliseconds.set(Calendar.MILLISECOND, 0);

		return dateWithoutMilliseconds;

	}		

	/* getIntervalDates returns a collection of days */

	public final static Collection getIntervalDates(Interval interval) {

		Collection <Calendar> dates = new ArrayList <Calendar> ();		

		long start = (interval.getStart()).getTime().getTime() / 1000;
		long end = (interval.getEnd()).getTime().getTime() / 1000;
		
		long days = ((end - start) / 86400) + 1; //1 Day = 86400 seconds
		
		Calendar currentDate = (Calendar) (interval.getStart()).clone();
		dates.add((Calendar) currentDate.clone());
		
		for (int i=1; i < days; i++) {
			currentDate.add(Calendar.DATE, 1);
			dates.add((Calendar) currentDate.clone());
		}
		
		return dates;

	}

	/* getIntervalHours returns a collection of hours */

	public final static Collection getIntervalHours(Interval interval) {

		Collection <Calendar> dates = new ArrayList <Calendar> ();
		dates.add(interval.getStart());

		long start = (interval.getStart()).getTime().getTime() / 1000;
		long end = ((interval.getEnd()).getTime().getTime()) + 1 / 1000;
		
		long hours = (end - start) / 3600; //1 Hour = 3600 seconds

		Calendar currentDate = Calendar.getInstance();
		currentDate.setTime(interval.getStart().getTime());

		for (int i=1; i<hours; i++) {
			(currentDate).add(Calendar.HOUR_OF_DAY, 1);
			dates.add(currentDate);
		}

		return dates;

	}

	public final static long getIntervalDifference(Interval interval) {

		long start = (interval.getStart()).getTime().getTime() / 1000;
		long end = (interval.getEnd()).getTime().getTime() / 1000;
		long difference = ((end - start) / 86400); //1 Day = 86400 seconds
		return difference;

	}

	public final static Calendar toCalendar(Timestamp dateAsTimestamp) {

		Calendar date = Calendar.getInstance();

		date.clear();
		date.setTimeInMillis(dateAsTimestamp.getTime());

		return date;

	}

	public final static Timestamp toTimeStamp(Calendar date) {
		return new Timestamp(date.getTimeInMillis());
	}

	public final static Calendar textToCalendar(String text) {				

		int day = new Integer(text.substring(0,2)).intValue();
		int month = new Integer(text.substring(3,5)).intValue();
		int year = new Integer(text.substring(6,10)).intValue();

		Calendar calendar = Calendar.getInstance();
		calendar.set(year, month - 1, day, 0, 0, 0);
		calendar.setTime(calendar.getTime());
		

		return calendar;

	}
	
	public final static String calendarToText(Calendar calendar) {
		String text = StringUtils.padLeft(Integer.toString(calendar.get(Calendar.DAY_OF_MONTH)),"0",2) + "" 
		+ StringUtils.padLeft(Integer.toString(calendar.get(Calendar.MONTH) + 1), "0", 2) + ""
		+ calendar.get(Calendar.YEAR);
		return text;
	}

	public final static boolean dateValidation(String text) {

		int day = new Integer(text.substring(0,2)).intValue();		
		int month = new Integer(text.substring(3,5)).intValue();
		int year = new Integer(text.substring(6,10)).intValue();

		try {
		
		GregorianCalendar gc = new GregorianCalendar();
		gc.setLenient(false);        // must do this
		gc.set(GregorianCalendar.YEAR, year);
		gc.set(GregorianCalendar.MONTH, month);
		gc.set(GregorianCalendar.DATE, day);

		gc.getTime();
		
		return true;
		
		} catch (Exception e) {
			return false;	
		}
		
	}

	public final static boolean dateValidation(Calendar date) {		

		try {
		
		GregorianCalendar gc = new GregorianCalendar();
		gc.setLenient(false);        // must do this
		gc.set(GregorianCalendar.YEAR, date.get(Calendar.YEAR));
		gc.set(GregorianCalendar.MONTH, date.get(Calendar.MONTH));
		gc.set(GregorianCalendar.DATE, date.get(Calendar.DAY_OF_MONTH));

		gc.getTime();
		
		return true;
		
		} catch (Exception e) {
			return false;	
		}
		
	}

	public final static Calendar getDateWithoutClock(Calendar date) {
		
		Calendar dateWithoutClock = Calendar.getInstance();
		
		dateWithoutClock.set(Calendar.HOUR, 0);
		dateWithoutClock.set(Calendar.MINUTE, 0);
		dateWithoutClock.set(Calendar.SECOND, 0);
		dateWithoutClock.set(Calendar.MILLISECOND, 0);
		
		dateWithoutClock.setTime(date.getTime());

		return dateWithoutClock;

	}
	
	public final static boolean sameDay(Calendar d1, Calendar d2) {

		if (d1.get(Calendar.DATE) == d2.get(Calendar.DATE) &&
				d1.get(Calendar.MONTH) == d2.get(Calendar.MONTH) &&
				d1.get(Calendar.YEAR) == d2.get(Calendar.YEAR))
			return true;
		
		return false;
		
	}
	
}



